<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Cms\Model\ResourceModel\Block\CollectionFactory as BlockCollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Store\Model\StoreManagerInterface;

class CmsBlock
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var Mongo
     */
    private $mongo;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var BlockCollectionFactory
     */
    protected $blockCollectionFactory;

    /**
     * Menu constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     * @param BlockCollectionFactory $blockCollectionFactory
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager,
        BlockCollectionFactory $blockCollectionFactory
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->scopeConfig = $scopeConfig;
        $this->mongo = $mongo;
        $this->storeManager = $storeManager;
        $this->blockCollectionFactory = $blockCollectionFactory;
    }

    public function exportCmsBlock()
    {
        foreach ($this->storeManager->getStores() as $key => $value) {
            $this->mongo->deleteDocumentsByProperties(
                $this->scopeConfig->getValue('mongo/general/mongo_general_database'),
                "contents_" . $value->getCode(),
                ['metacode' => 'block']
            );

            $collection = $this->blockCollectionFactory->create();
            $collection->addStoreFilter($value)->addFilter('is_active', 1);

            $bulk = [];
            foreach ($collection as $item) {
                $block = new \stdClass();

                $block->_id = "block_" . $item->getId() . "_" . $value->getData('code');
                $block->metacode = "block";
                $block->store = $value->getData('code');
                $block->content_id = (int)  $item->getId();
                $block->url_key = $item->getIdentifier();
                $block->idx = $item->getIdentifier();
                $block->title = $item->getTitle();
                $block->type = "static_block";
                $block->status = $item->isActive();
                $block->content = $item->getContent();
                $block->display_entity = $item->getDisplayOn();
                $block->display_selection_type = $item->getSelectionType();
                $block->display_selection_target = $this->parseTargetSelection($item->getSelectionTarget());
                $block->display_container = $item->getRefiloLayoutContainer();
                $block->display_sort_order = (int)$item->getRefiloLayoutOrder();
                $block->listing_display_page = (int)$item->getListingDisplayPage();
                $block->listing_display_position = (int)$item->getListingDisplayPosition();
                $block->listing_columns_spacing = (int)$item->getListingColumnsSpacing();

                $bulk[] = $block;
            }

            try {
                $this->mongo->update(
                    '_id',
                    $bulk,
                    "contents_" . $value->getCode(),
                    true,
                    $this->scopeConfig->getValue('mongo/general/mongo_general_database')
                );
            } catch (MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $value
     * @return false|string[]
     */
    protected function parseTargetSelection($value)
    {
        $strippedValue = preg_replace('/\s/', '', $value);
        return explode(',', $strippedValue);
    }
}
